# Generated by Django 5.0 on 2023-12-22 09:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("emodjis", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="emoji",
            name="nsfw",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="emoji",
            name="team",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="emojis",
                to="auth.group",
            ),
        ),
    ]
