<div class="float-right">
    {% for p in page_range %}<a href="#" unicorn:click="page={{ p }}" unicorn:model="page" class="text-lg underline p-1">{% if p == page %}<b>{{ p }}</b>{% else %}{{ p }}{% endif %}</a> {% endfor %}
</div>
