{% extends "base.html" %}
{% load django_bootstrap5 %}
{% load unicorn %}

{% block title %}{{ title }}{% endblock %}
{% block content_title %}<a href="/" class="tracking-wide text-white text-center"><b>{{ title }}</b></a>{% endblock %}

{% block content %}
    {% csrf_token %}
    {% unicorn 'emoji-grid' id='emoji-grid' key='emoji-grid' %}
{% endblock %}
